@@ -13,7 +13,8 @@ program SEQ
 
 !-- namelist valiables
   integer :: nr, nz
-  real :: dr, dz
+  integer :: mom_flag
+  real :: coril
   character(4) :: bc
   character(80) :: fname
   character(80) :: sound_name  ! ǥ󥰥ե̾
@@ -74,7 +75,7 @@ program SEQ
   real :: coriol
 
 !-- namelist Υѥ᡼
-  namelist /input /nr, nz, dr, dz, bc, fname, sound_name
+  namelist /input /nr, nz, bc, fname, sound_name, coril, mom_flag
   read(5,nml=input)
 
 !-- γ
@@ -123,12 +124,12 @@ program SEQ
   allocate(tauz(nr,nz))
   allocate(nu(nr,nz))
 
-  coriol=2.0*omega*sin(20.0*pi/180.0)
+  coriol=2.0*omega*sin(coril*pi/180.0)
   bound=0.0
 
 !-- ɸͤ
-  r=(/(dr*real(i-1),i=1,nr)/)
-  z=(/(dz*real(i-1),i=1,nz)/)
+!  r=(/(dr*real(i-1),i=1,nr)/)
+!  z=(/(dz*real(i-1),i=1,nz)/)
 
 !-- ͤɤ߹
 
@@ -142,72 +143,93 @@ program SEQ
   call HistoryGet( trim(fname), 'r', r(1:nr) )
   call HistoryGet( trim(fname), 'z', z(1:nz) )
   call HistoryGet( trim(fname), 'pres', pres(1:nr,1:nz) )
-  call HistoryGet( trim(fname), 'qv', Q(1:nr,1:nz) )
-  call HistoryGet( trim(fname), 'theta0', theta0(1:nr,1:nz) )
+  call HistoryGet( trim(fname), 'Q', Q(1:nr,1:nz) )
+  call HistoryGet( trim(fname), 'theta', theta0(1:nr,1:nz) )
   call HistoryGet( trim(fname), 'v', v0(1:nr,1:nz) )
   call HistoryGet( trim(fname), 'rho', rho(1:nr,1:nz) )
 
-  mom=0.0
-  dmomdz=0.0
-  nu=0.0
-
-  do i=1,nr
-     call grad_1d( z, v0(i,:), tauz(i,:) )
-  end do
-
-  do j=1,nz
-     call grad_1d( r, v0(:,j), taur(:,j) )
-  end do
-
-  do j=1,nz
-     do i=2,nr
-        nu(i,j)=200.0**2*sqrt((taur(i,j)-v0(i,j)/r(i))**2+tauz(i,j)**2)
-     end do
-  end do
-
-  do j=1,nz
-     do i=2,nr
-        if(j/=1)then
-           taur(i,j)=nu(i,j)*(taur(i,j)-v0(i,j)/r(i))
-        else
-           taur(i,j)=-Cd*abs(v0(i,j))*v0(i,j)
-        end if
-     end do
-  end do
-
+!-- Q ǮΨºݤη׻ɬפϤ˴.
   do j=1,nz
      do i=1,nr
-        tauz(i,j)=nu(i,j)*tauz(i,j)
+        Q(i,j)=Q(i,j)*g/Cpd/theta0(i,j)
      end do
   end do
 
+!-- ư̥η׻
+
+  mom=0.0
+
+  select case (mom_flag)
+  case (0)
+     write(*,*) "### momentum source is zero."
+  case (1)  ! ®پ줫׻.
+     write(*,*) "### momentum source is calculating with Vt."
+     dmomdz=0.0
+     nu=0.0
+
+     do i=1,nr
+        call grad_1d( z, v0(i,:), tauz(i,:) )
+     end do
+
+     do j=1,nz
+        call grad_1d( r, v0(:,j), taur(:,j) )
+     end do
+
+     do j=1,nz
+        do i=2,nr
+           nu(i,j)=200.0**2*sqrt((taur(i,j)-v0(i,j)/r(i))**2+tauz(i,j)**2)
+        end do
+     end do
+
+     do j=1,nz
+        do i=2,nr
+           if(j/=1)then
+              taur(i,j)=nu(i,j)*(taur(i,j)-v0(i,j)/r(i))
+           else
+              taur(i,j)=-Cd*abs(v0(i,j))*v0(i,j)
+           end if
+        end do
+     end do
+
+     do j=1,nz
+        do i=1,nr
+           tauz(i,j)=nu(i,j)*tauz(i,j)
+        end do
+     end do
+
 !-- ޤǤ, ϥƥ󥽥η׻λ.
 !-- ʹߤήեåη׻.
 
-  do i=1,nr
-     call grad_1d( z, tauz(i,:), dmomdz(i,:) )
+     do i=1,nr
+        call grad_1d( z, tauz(i,:), dmomdz(i,:) )
+        do j=1,nz
+           mom(i,j)=dmomdz(i,j)
+        end do
+     end do
+
      do j=1,nz
-        mom(i,j)=dmomdz(i,j)
+        call grad_1d( r, taur(:,j), dmomdz(:,j) )
+        do i=1,nr
+           mom(i,j)=mom(i,j)+dmomdz(i,j)
+        end do
+        do i=2,nr
+           mom(i,j)=mom(i,j)+2.0*taur(i,j)/r(i)
+        end do
+        do i=1,nr
+           mom(i,j)=mom(i,j)/rho(i,j)
+        end do
      end do
-  end do
 
-  do j=1,nz
-     call grad_1d( r, taur(:,j), dmomdz(:,j) )
-     do i=1,nr
-        mom(i,j)=mom(i,j)+dmomdz(i,j)
-     end do
-     do i=2,nr
-        mom(i,j)=mom(i,j)+2.0*taur(i,j)/r(i)
-     end do
-     do i=1,nr
-        mom(i,j)=mom(i,j)/rho(i,j)
-     end do
-  end do
+!     do i=1,nr
+!        mom(i,1)=-Cd*abs(v0(i,1))*v0(i,1)
+!        call grad_1d( z, mom(i,:), dmomdz(i,:) )
+!     end do
 
-!  do i=1,nr
-!     mom(i,1)=-Cd*abs(v0(i,1))*v0(i,1)
-!     call grad_1d( z, mom(i,:), dmomdz(i,:) )
-!  end do
+  case (2)
+     write(*,*) "### momentum source is writing the external file."
+     call HistoryGet( trim(fname), 'mom', mom(1:nr,1:nz) )
+
+  end select
 
 !-- ʪ̤η׻
   do j=1,nz
@@ -302,7 +324,7 @@ end do
 
 !-- ݥ󥽥
 
-  call Poisson_GauSei( r(1:nr), z(1:nz), force(1:nr,1:nz), eps,  &
+  call Poisson_Jacobi( r(1:nr), z(1:nz), force(1:nr,1:nz), eps,  &
   &                    bc, psi(1:nr,1:nz), bound(1:nr,1:nz),  &
   &                    a=term1(1:nr,1:nz),  b=term2(1:nr,1:nz),  &
   &                    c=term3(1:nr,1:nz),  d=term4(1:nr,1:nz),  &
