@@ -163,13 +163,6 @@ write(*,*) rho_s(i)
 
   call hydro_grad_eqb( r, z, coril, v0, pres_s, rho_s, pres, rho )
 
-do j=1,nz
-do i=1,nr
-if(rho(i,j)<=0.0)then
-write(*,*) "solv afte", rho(i,j), i, j
-end if
-end do
-end do
 !-- Фɬפ pt ׻
 
   do j=1,nz
@@ -196,6 +189,7 @@ end do
   do j=1,nz
      do i=1,nr
         qv(i,j)=max_val*func_a(zd(j))*Bell( m_bell, x(i,j) )
+        qv(i,j)=qv(i,j)*Cpd*theta0(i,j)/g
      end do
   end do
 
@@ -215,38 +209,38 @@ end do
     & longnames=(/'r-coordinate','z-coordinate'/),  &
     & units=(/'m','m'/), origin=0.0, interval=0.0 )
 
-write(*,*) "########"
   call HistoryPut( 'r', r )
   call HistoryPut( 'z', z )
-write(*,*) "########"
 
   call HistoryAddVariable( varname='v', dims=(/'r','z'/), &
     & longname='tangential wind', units='m/s', xtype='float')
 
-write(*,*) "########"
   call HistoryPut( 'v',v0 )
 
-write(*,*) "########"
   call HistoryAddVariable( varname='pres', dims=(/'r','z'/), &
     & longname='pressure', units='Pa', xtype='float')
 
   call HistoryPut( 'pres',pres )
 
-write(*,*) "########"
-  call HistoryAddVariable( varname='theta0', dims=(/'r','z'/), &
+  call HistoryAddVariable( varname='theta', dims=(/'r','z'/), &
     & longname='potential temperature', units='K', xtype='float')
 
-  call HistoryPut( 'theta0', theta0 )
+  call HistoryPut( 'theta', theta0 )
 
   call HistoryAddVariable( varname='rho', dims=(/'r','z'/), &
     & longname='density', units='kg/m3', xtype='float')
 
   call HistoryPut( 'rho', rho )
 
-  call HistoryAddVariable( varname='qv', dims=(/'r','z'/), &
+  call HistoryAddVariable( varname='Q', dims=(/'r','z'/), &
     & longname='heating rate', units='J/s', xtype='float')
 
-  call HistoryPut( 'qv',qv )
+  call HistoryPut( 'Q',qv )
+
+  call HistoryAddVariable( varname='mom', dims=(/'r','z'/), &
+    & longname='heating rate', units='m/s2', xtype='float')
+
+  call HistoryPut( 'Q',qv )
 
   call HistoryClose
 
