@@ -27,6 +27,7 @@ program sound_1d
   real, allocatable, dimension(:) :: ww      ! 
   real, allocatable, dimension(:) :: tmp     ! ƥݥ
   real, allocatable, dimension(:,:) :: draw_tate, draw_yoko, write_data
+  real, dimension(11) :: col_lev  ! ۥɥչ
   real :: height_lcl, pres_lcl  ! LCL 
   real :: height_lfc, pres_lfc  ! LFC 
   real :: height_lnb, pres_lnb  ! LNB 
@@ -35,29 +36,32 @@ program sound_1d
   real :: precip  ! Ĺ߿
   real :: z_ref  ! CAPE η׻κ, .
   real :: p_ref  ! CAPE η׻κ, ൤.
+  real :: unit_v  ! ñ̥٥ȥ礭.
   real, dimension(2) :: draw_region_p, draw_region_z
+  real, dimension(2) :: vx_new, vy_new
   real, dimension(6), parameter :: convx=(/0.3, 0.5, 0.55, 0.75, 0.55, 0.75/),  &
                                    convy=(/0.5, 0.8, 0.5, 0.8, 0.5, 0.8/)
   real, allocatable, dimension(:,:) :: cvx, cvy
 
-  integer :: i, j, k, l, m, n, iz, jmax
+  integer :: i, j, k, l, m, n, iz, jmax, idz_min, idz_max
   integer :: nz  ! 
   integer :: nnz  ! ǡޤȤ˻Ȥΰˤ
   integer :: nx  ! ǡο
   integer :: IWS
   integer, parameter :: fig_num=3  ! ޤμ
+  integer, dimension(11) :: col_num  ! ۥɥչ
 
   character(80), allocatable, dimension(:,:) :: fname  ! ǡե̾
   character(80) :: list_name  ! ǡꥹȥե̾
   character(20), allocatable, dimension(:,:) :: cval  ! ǡɤ߹
   character(1000) :: tmp_char
-  character(4) :: draw_flag  ! ޤ˴ؤե饰
+  character(5) :: draw_flag  ! ޤ˴ؤե饰
   character(10) :: t_c
   character(28) :: skew_c
   character(10), allocatable, dimension(:) :: title_c, title_u
   character(1) :: refer_flag
 
-  logical :: dmp_flag
+  logical :: dmp_flag, no_draw_flag
 
   real :: undef
   integer, parameter :: skip_num=2
@@ -65,7 +69,7 @@ program sound_1d
 !-- namelist file ɤ߹
 
   namelist /input /z_ref, p_ref, list_name, dmp_flag, undef, draw_flag,  &
-  &                draw_region_p, draw_region_z, IWS, refer_flag
+  &                draw_region_p, draw_region_z, IWS, refer_flag, unit_v
   read(5,nml=input)
 
   if(z_ref==-999.0)then
@@ -76,6 +80,14 @@ program sound_1d
      refer_flag='z'
   end if
 
+  no_draw_flag=.true.
+  do i=1,len_trim(draw_flag)
+     if(draw_flag(i:i)=='o')then
+        no_draw_flag=.false.
+        exit
+     end if
+  end do
+
   jmax=11
 
   allocate(title_c(jmax))
@@ -97,6 +109,11 @@ program sound_1d
      end do
   end do
 
+!-- ۥɥչѥ顼.
+  call color_setting( 10, draw_region_z, col_min=15, col_max=85 )
+  col_num=(/((153+7*(i-1)*10),i=1,11)/)
+  col_lev=(/((draw_region_z(1)+((draw_region_z(2)-draw_region_z(1))/10.0)*real(i-1)),i=1,11)/)
+
 !-- ɤ߹ǡեο
   nnz=line_number_counter( trim(list_name) )
 
@@ -104,7 +121,9 @@ program sound_1d
   allocate(write_data(nnz,jmax))
 
 !-- DCL set
-  call DclOpenGraphics(IWS)
+  if(no_draw_flag.eqv..false.)then
+     call DclOpenGraphics(IWS)
+  end if
 
   call GLRSET('RMISS',undef)
   call GLLSET('LMISS',.true.)
@@ -133,8 +152,8 @@ program sound_1d
      allocate(pt(nz))
      allocate(ept(nz))
      allocate(sept(nz))
-     allocate(draw_tate(nz,3))
-     allocate(draw_yoko(nz,3))
+     allocate(draw_tate(nz,4))
+     allocate(draw_yoko(nz,4))
 
 !-- ե뤫ǡɤ߹
      call read_file_text( trim(fname(1,i)), 6, nz, cval, skip=skip_num )
@@ -183,7 +202,8 @@ program sound_1d
 
 !-- ή԰ѥ᡼׻
 !-- 1. CAPE ׻
-     cape_c=CAPE( pres, height, qv, temp, z_ref, undef )
+     cape_c=CAPE( pres, height, qv, temp, z_ref, undeff=undef )!, opt=2 )
+!     write(*,*) "MESSAGE : CAPE option is 2."
 
 !-- 2. CIN ׻
      cin_c=CIN( pres, height, qv, temp, z_ref, undef )
@@ -258,17 +278,42 @@ program sound_1d
         end do
      end if
 
+!-- ۥɥʬۺʬ
+     if(draw_flag(5:5)=='o')then
+        do j=1,nz
+           draw_tate(j,1)=height(j)
+           if(ew(j)/=undef.and.ww(j)/=undef)then
+              draw_yoko(j,4)=ew(j)
+              draw_tate(j,4)=ww(j)
+           else
+              draw_yoko(j,4)=undef
+              draw_tate(j,4)=undef
+           end if
+        end do
+     end if
+
 !-- skew-T ץåȺʬ
 
-     call SGLSET('LCLIP',.true.)
-
      if(draw_flag(1:1)=='o')then
-        call Dcl_PL( 'l', trim(fname(2,i)), draw_yoko, draw_tate,  &
-  &                  draw_yoko, draw_tate,  &
-  &                  (/achar(241)//'(pink), '//achar(241)//'_e"(blue), '//  &
-  &                    achar(241)//'_es"(orange)  (K)', 'height (m)'/),  &
-  &                  x_int=(/280.0, 380.0/), y_int=(/0.0, 2.0e4/),  &
-  &                  viewx_int=(/0.3, 0.7/), viewy_int=(/0.2, 0.8/) )
+        call DclSetParm( 'GRAPH:LCLIP', .true. )
+        call Dcl_PL( 'l', trim(fname(2,i)),  &
+  &                  draw_yoko(:,1:3), draw_tate(:,1:3),  &
+  &                  draw_yoko(:,1:3), draw_tate(:,1:3),  &
+  &                  (/achar(241)//'(black), '//achar(241)//'_e"(red), '//  &
+  &                    achar(241)//'_es"(green)  (K)', 'height (m)'/),  &
+  &                  x_int=(/280.0, 380.0/), y_int=draw_region_z,  &
+  &                  viewx_int=(/0.3, 0.7/), viewy_int=(/0.2, 0.8/),  &
+  &                  l_idx=(/14,24,34/) )
+        if(draw_flag(3:3)=='o')then
+           call DclSetParm( 'GRAPH:LCLIP', .false. )
+           CALL SGQVPT( vx_new(1), vx_new(2), vy_new(1), vy_new(2) )
+           call axis_vec( draw_region_z, vx_new(2)+0.01, pres, ew, ww,  &
+  &                       sondeZ=height, undef=undef, vn=20,  &
+  &                       view_int=vy_new,  &
+  &                       unit_flag=.true., unity=unit_v,  &
+  &                       u_posi=(/vx_new(2)+0.1,vy_new(1)-0.025/),  &
+  &                       uni_len=0.05 )
+        end if
         if(draw_flag(4:4)=='o')then
            call Dcl_Square_Normal( convx(1:2), convy(1:2), 12, 999999 )
            do j=1,jmax
@@ -285,9 +330,18 @@ program sound_1d
      end if
 
      if(draw_flag(2:2)=='o')then
+        call DclSetParm( 'GRAPH:LCLIP', .true. )
         call SkewT( trim(fname(2,i)), 20.0, (/240.0, 300.0/),  &
-  &                 draw_region_p, temp, pres, qv,  &
-  &                 undef=-999.0 )
+  &                 draw_region_p, temp, pres, qv, undef=undef )
+        if(draw_flag(3:3)=='o')then
+           call DclSetParm( 'GRAPH:LCLIP', .false. )
+           CALL SGQVPT( vx_new(1), vx_new(2), vy_new(1), vy_new(2) )
+           call axis_vec( draw_region_p, vx_new(2)+0.01, pres, ew, ww,  &
+  &                       undef=undef, view_int=vy_new,  &
+  &                       unit_flag=.true., unity=unit_v,  &
+  &                       u_posi=(/vx_new(2)+0.1,vy_new(1)-0.025/),  &
+  &                       uni_len=0.05 )
+        end if
         if(draw_flag(4:4)=='o')then
            call Dcl_Square_Normal( convx(3:4), convy(3:4), 12, 999999 )
            do j=1,jmax
@@ -303,25 +357,30 @@ program sound_1d
         end if
      end if
 
-     if(draw_flag(3:3)=='o')then
-        call SkewT_vec( trim(fname(2,i)), 20.0, (/240.0, 300.0/),  &
-  &                     draw_region_p, temp, pres, qv, ew, ww,  &
-  &                     undef=-999.0 )
-        if(draw_flag(4:4)=='o')then
-           call Dcl_Square_Normal( convx(5:6), convy(5:6), 12, 999999 )
-           do j=1,jmax
-              if(trim(adjustl(title_u(j)))=='hPa'.and.write_data(i,j)/=undef)then
-                 skew_c=trim(title_c(j))//r2c_convert( write_data(i,j)*1.0e-2, '(f8.1)' )
-              else
-                 skew_c=trim(title_c(j))//r2c_convert( write_data(i,j), '(f8.1)' )
-              end if
-              skew_c=trim(skew_c)//trim(title_u(j))
-              call DclDrawTextNormalized( cvx(j,3), cvy(j,3), trim(skew_c),  &
-  &                                    height=0.0125, centering=-1 )
-           end do
-        end if
-     end if
+!-- ۥɥʬ
 
+     if(draw_flag(5:5)=='o')then
+        call interpo_search_1d( height, draw_region_z(1), idz_min )
+        if(idz_min<1)then
+           idz_min=1
+        end if
+        call interpo_search_1d( height, draw_region_z(2), idz_max )
+
+        call Dcl_PL( 'l', trim(fname(2,i)),  &
+  &                  draw_yoko(idz_min:idz_max,4:4),  &
+  &                  draw_tate(idz_min:idz_max,4:4),  &
+  &                  draw_yoko(idz_min:idz_max,4:4),  &
+  &                  draw_tate(idz_min:idz_max,4:4),  &
+  &                  (/'Zonal Wind', 'Meridional Wind'/),  &
+  &                  cl_val=col_lev, cl_idx=col_num,  &
+  &                  zline=draw_tate(idz_min:idz_max,1:1) )
+                     ! draw_tate Ͼ Ema ǹ٤ȤƻȤäƤ.
+        CALL SGQVPT( vx_new(1), vx_new(2), vy_new(1), vy_new(2) )
+        call tone_bar( 10, draw_region_z,  &
+  &                    (/vx_new(2)+0.025, vx_new(2)+0.05/), vy_new,  &
+  &                    '(f8.2)', col_mem_num=10,  &
+  &                    title='height [m]', titles='t', titlep=0.0 )
+     end if
 
      deallocate(cval)
      deallocate(height)
@@ -344,6 +403,10 @@ program sound_1d
   end if
   end do
 
+  if(no_draw_flag.eqv..false.)then
+     call DclCloseGraphics
+  end if
+
 !-- 10. Ƽѥ᡼Υƥȥǡ¸
 
   if(dmp_flag.eqv..true.)then  ! ήѥ᡼ƥȥǡȤ¸.
@@ -387,6 +450,4 @@ write(*,*) "format", trim(tmp_char)
 
   end if
 
-  call DclCloseGraphics
-
 end program
