@@ -16,6 +16,7 @@ file = Array[
 "poly_function.f90",
 "special_function.f90",
 "statistics.f90",
+"stdio.f90",
 "thermo_advanced_function.f90",
 "thermo_advanced_routine.f90",
 "thermo_const.f90",
@@ -42,6 +43,7 @@ dir = Array[
 "Poly_Function",
 "Special_Function",
 "Statistics",
+"stdio",
 "Thermo_Advanced_Function",
 "Thermo_Advanced_Routine",
 "Thermo_Const",
