@@ -76,7 +76,7 @@ subroutine Mean_1d( x, ave, error )  ! 1 ʿͷ׻롼
      if(nt/=0)then
         ave=summ/nt
      else
-        ave=0.0
+        ave=error
      end if
 
   else
@@ -120,7 +120,7 @@ subroutine Mean_2d( x, ave, error )  ! 2 ʿͷ׻롼
      if(nt/=0)then
         ave=summ/nt
      else
-        ave=0.0
+        ave=error
      end if
 
   else
@@ -170,7 +170,7 @@ subroutine Mean_3d( x, ave, error )  ! 3 ʿͷ׻롼
      if(nt/=0)then
         ave=summ/nt
      else
-        ave=0.0
+        ave=error
      end if
 
   else
@@ -605,7 +605,7 @@ subroutine nearest_search_3d( x, y, z, pointx, pointy, pointz, i, j, k )
 end subroutine nearest_search_3d
 
 
-subroutine interpo_search_1d( x, point, i, undeff )
+subroutine interpo_search_1d( x, point, i, undeff, stdopt )
   ! ǿ뤴Ȥͤ礭ʤˤΤʤ,
   ! point ֹϤ.
   implicit none
@@ -613,27 +613,38 @@ subroutine interpo_search_1d( x, point, i, undeff )
   real, intent(in) :: point  ! 
   integer, intent(inout) :: i  ! point ͤۤʤֹͤ
   integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
+                                           ! default Ǥ .false. (ɽ)
   integer :: nx, j
   integer :: just
+  logical :: stderr
 
   nx=size(x)
   if(present(undeff))then
-     just=int(undeff)
+     just=undeff
   else
-     just=0
+     just=-2147483648
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
   end if
 
   do j=1,nx
      if(x(1)>point)then
-        write(*,*) "****** WARNING ******"
-        write(*,*) "searching point was not found."
-        write(*,*) "Abort. Exit.!!!"
+        if(stderr.eqv..false.)then
+           write(*,*) "****** WARNING ******"
+           write(*,*) "searching point was not found :", x(1), point
+           write(*,*) "Abort. Exit.!!!"
+        end if
         i=just
         exit
      end if
 
      if(present(undeff))then
-        if(x(j)/=undeff)then
+        if(x(j)/=real(undeff))then
            if(x(j)<=point)then
               i=j
            else
@@ -652,7 +663,7 @@ subroutine interpo_search_1d( x, point, i, undeff )
 end subroutine interpo_search_1d
 
 
-subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff )
+subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff, stdopt )
   ! ǿ뤴Ȥͤ礭ʤˤΤʤ,
   ! point ֹϤ.
   implicit none
@@ -663,21 +674,30 @@ subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff )
   integer, intent(inout) :: i  ! pointx ͤۤʤֹͤ
   integer, intent(inout) :: j  ! pointy ͤۤʤֹͤ
   integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: just
+  logical :: stderr
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
 
   if(present(undeff))then
-     just=int(undeff)
-     call interpo_search_1d( x, pointx, i, just )
-     call interpo_search_1d( y, pointy, j, just )
+     just=undeff
+     call interpo_search_1d( x, pointx, i, just, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, just, stdopt=stderr )
   else
-     call interpo_search_1d( x, pointx, i )
-     call interpo_search_1d( y, pointy, j )
+     call interpo_search_1d( x, pointx, i, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, stdopt=stderr )
   end if
 
 end subroutine interpo_search_2d
 
 
-subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
+subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k,  &
+  &                           undeff, stdopt )
   ! ǿ뤴Ȥͤ礭ʤˤΤʤ,
   ! point ֹϤ.
   implicit none
@@ -691,17 +711,25 @@ subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
   integer, intent(inout) :: j  ! pointy ͤۤʤֹͤ
   integer, intent(inout) :: k  ! pointz ͤۤʤֹͤ
   integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: just
+  logical :: stderr
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
 
   if(present(undeff))then
      just=int(undeff)
-     call interpo_search_1d( x, pointx, i, just )
-     call interpo_search_1d( y, pointy, j, just )
-     call interpo_search_1d( z, pointz, k, just )
+     call interpo_search_1d( x, pointx, i, just, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, just, stdopt=stderr )
+     call interpo_search_1d( z, pointz, k, just, stdopt=stderr )
   else
-     call interpo_search_1d( x, pointx, i )
-     call interpo_search_1d( y, pointy, j )
-     call interpo_search_1d( z, pointz, k )
+     call interpo_search_1d( x, pointx, i, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, stdopt=stderr )
+     call interpo_search_1d( z, pointz, k, stdopt=stderr )
   end if
 
 end subroutine interpo_search_3d
@@ -824,7 +852,7 @@ end subroutine interpolation_3d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_1d( x, r, u, v, undef )
+subroutine auto_interpolation_1d( x, r, u, v, undef, undefr, stdopt )
   ! ɸ x Ƥǡ u 
   ! ɸ r ǡ v ˼ưޤ.
   implicit none
@@ -832,9 +860,14 @@ subroutine auto_interpolation_1d( x, r, u, v, undef )
   real, intent(in) :: r(:)  ! ޺ɸ
   real, intent(in) :: u(size(x))  ! ǡ
   real, intent(inout) :: v(size(r))  ! ޤǡ
-  real, intent(in), optional :: undef  ! ̤
+  integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undefr  ! ΰǤ̤.
+        !  undefr ʤ,  undefr 
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, nx, nr, ir
-  real :: defun
+  integer :: defun
+  real :: rdefun
+  logical :: stderr
 
   nx=size(x)
   nr=size(r)
@@ -845,13 +878,29 @@ subroutine auto_interpolation_1d( x, r, u, v, undef )
      defun=-2147483648
   end if
 
+  if(present(undefr))then
+     rdefun=undefr
+  else
+     rdefun=-999.0
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
+
   do i=1, nr
-     call interpo_search_1d( x, r(i), ir, int(defun) )
+     call interpo_search_1d( x, r(i), ir, int(defun), stdopt=stderr )
 
      if(ir/=int(defun).and.ir<nx)then
-        call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
+        if(u(ir)/=rdefun.and.u(ir+1)/=rdefun)then
+           call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
+        else
+           v(i)=rdefun
+        end if
      else
-        v(i)=defun
+        v(i)=real(defun)
      end if
   end do
 
@@ -860,7 +909,7 @@ end subroutine auto_interpolation_1d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
+subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, undefr, stdopt )
   ! ɸ x, y Ƥǡ u 
   ! ɸ r, q ǡ v ˼ưޤ.
   implicit none
@@ -870,9 +919,14 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
   real, intent(in) :: q(:)  ! ޺ɸ 2
   real, intent(in) :: u(size(x),size(y))  ! ǡ
   real, intent(inout) :: v(size(r),size(q))  ! ޤǡ
-  real, intent(in), optional :: undef  ! ̤
+  integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undefr  ! ΰǤ̤.
+        !  undefr ʤ,  undefr 
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, nx, ny, nr, nq, ir, iq
-  real :: defun
+  integer :: defun
+  real :: rdefun
+  logical :: stderr
 
   nx=size(x)
   ny=size(y)
@@ -885,15 +939,33 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
      defun=-2147483648
   end if
 
+  if(present(undefr))then
+     rdefun=undefr
+  else
+     rdefun=-999.0
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
+
   do j=1, nq
      do i=1, nr
-        call interpo_search_2d( x, y, r(i), q(j), ir, iq, int(defun) )
+        call interpo_search_2d( x, y, r(i), q(j), ir, iq, int(defun),  &
+  &                             stdopt=stderr )
 
         if(ir/=int(defun).and.iq/=int(defun).and.ir<nx.and.iq<ny)then
-           call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
-  &                               u(ir:ir+1,iq:iq+1), (/r(i), q(j)/), v(i,j) )
+           if(u(ir,iq)/=rdefun.and.u(ir+1,iq)/=rdefun.and.  &
+  &           u(ir,iq+1)/=rdefun.and.u(ir+1,iq+1)/=rdefun)then
+              call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
+  &                              u(ir:ir+1,iq:iq+1), (/r(i), q(j)/), v(i,j) )
+           else
+              v(i,j)=rdefun
+           end if
         else
-           v(i,j)=defun
+           v(i,j)=real(defun)
         end if
      end do
   end do
@@ -903,7 +975,7 @@ end subroutine auto_interpolation_2d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
+subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, undefr, stdopt )
   ! ɸ x, y, z Ƥǡ u 
   ! ɸ r, q, p ǡ v ˼ưޤ.
   implicit none
@@ -915,9 +987,14 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
   real, intent(in) :: p(:)  ! ޺ɸ 3
   real, intent(in) :: u(size(x),size(y),size(z))  ! ǡ
   real, intent(inout) :: v(size(r),size(q),size(p))  ! ޤǡ
-  real, intent(in), optional :: undef  ! ̤
+  integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undefr  ! ΰǤ̤.
+        !  undefr ʤ,  undefr 
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, k, nx, ny, nz, nr, nq, np, ir, iq, ip
-  real :: defun
+  integer :: defun
+  real :: rdefun
+  logical :: stderr
 
   nx=size(x)
   ny=size(y)
@@ -932,19 +1009,38 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
      defun=-2147483648
   end if
 
+  if(present(undefr))then
+     rdefun=undefr
+  else
+     rdefun=-999.0
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
+
   do k=1, np
      do j=1, nq
         do i=1, nr
            call interpo_search_3d( x, y, z, r(i), q(j), p(k),  &
-  &                                ir, iq, ip, int(defun) )
+  &                                ir, iq, ip, int(defun), stdopt=stderr )
 
            if(ir/=int(defun).and.iq/=int(defun).and.ip/=int(defun).and.  &
   &           ir<nx.and.iq<ny.and.ip<nz)then
-              call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
-  &                               u(ir:ir+1,iq:iq+1,ip:ip+1),  &
-  &                               (/r(i), q(j), p(k)/), v(i,j,k) )
+              if(u(ir,iq,ip)/=rdefun.and.u(ir+1,iq,ip)/=rdefun.and.  &
+  &              u(ir,iq+1,ip)/=rdefun.and.u(ir+1,iq+1,ip)/=rdefun.and.  &
+  &              u(ir,iq,ip+1)/=rdefun.and.u(ir+1,iq,ip+1)/=rdefun.and.  &
+  &              u(ir,iq+1,ip+1)/=rdefun.and.u(ir+1,iq+1,ip+1)/=rdefun)then
+                 call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
+  &                                  u(ir:ir+1,iq:iq+1,ip:ip+1),  &
+  &                                  (/r(i), q(j), p(k)/), v(i,j,k) )
+              else
+                 v(i,j,k)=rdefun
+              end if
            else
-              v(i,j,k)=defun
+              v(i,j,k)=real(defun)
            end if
         end do
      end do
@@ -973,12 +1069,14 @@ subroutine LSM_1d( x, y, slope, intercept, undef )  ! Ǿˡˤ뷹ҷ׻ (1 ǡ)
   c=0.0
   d=0.0
 
-!$omp parallel do shared(u, v, x, y) private(i)
+!$omp parallel default(shared)
+!$omp do private(i)
   do i=1,nx
      u(i)=x(i)*x(i)
      v(i)=x(i)*y(i)
   end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
 
   if(present(undef))then
      call summ(v,a,undef)
@@ -1465,7 +1563,356 @@ end subroutine Cor_Coe_3d
 !---------------------------------
 !---------------------------------
 
-subroutine Move_ave( x, n, y, error )
+subroutine smooth_1d( x, n, y, method, weight, error )
+! 1 ǡˤĤ, ࡼ󥰤롼
+! , error ץϵǽƤʤ.
+  implicit none
+  real, intent(in) :: x(:)  ! ࡼ󥰤ǡ
+  integer, intent(in) :: n  ! ࡼαƶʻҿ(濴ޤ᤿)
+  real, intent(inout) :: y(size(x))  ! ࡼ󥰤줿ǡ
+  character(3), intent(in) :: method  ! ࡼ󥰤ˡ
+                            ! "SMP" = ñʿ, "OPT" = ץŤ
+                            ! "MAX" = , "MIN" = Ǿ
+  real, intent(in), optional :: weight(n)  ! method  "OPT" ξ
+                            ! weight(1) ü, weight(n) üȤƽŤ
+  real, intent(in), optional :: error  ! ̤
+  integer :: ix, j, mx, half, val
+  real :: div_fact
+  real :: wg(n)
+
+  y=0.0
+  mx=size(x)
+  half=(n-1)/2
+
+  select case (method(1:3))
+  case ("SMP")
+     do j=1,n
+        wg(j)=1.0
+     end do
+
+  case ("OPT")
+     if(present(weight))then
+        do j=1,n
+           wg(j)=weight(j)
+        end do
+     end if
+  end select
+
+  if(method(1:3)/="MIN".and.method(1:3)/="MAX")then
+!-- determining dividing factor
+
+     div_fact=0.0
+
+     do j=1,n
+        if(wg(j)<0.0)then
+           div_fact=1.0
+           exit
+        else
+           div_fact=div_fact+wg(j)
+        end if
+     end do
+
+!-- avoiding zero dividing
+
+     if(div_fact==0.0)then
+        div_fact=1.0
+     end if
+
+     do ix=half+1,mx-half
+        do j=1,n
+           y(ix)=y(ix)+x(ix-half-1+j)*wg(j)
+        end do
+        y(ix)=y(ix)/div_fact
+     end do
+
+  else
+
+     select case (method(1:3))
+     case ("MAX")
+        do ix=half+1,mx-half
+           val=x(ix-half)
+           do j=2,n
+              if(val<x(ix-half-1+j))then
+                 val=x(ix-half-1+j)
+              end if
+           end do
+           y(ix-half)=val
+        end do
+
+     case ("MIN")
+        do ix=half+1,mx-half
+           val=x(ix-half)
+           do j=2,n
+              if(val>x(ix-half-1+j))then
+                 val=x(ix-half-1+j)
+              end if
+           end do
+           y(ix-half)=val
+        end do
+
+     end select
+
+  end if
+
+end subroutine smooth_1d
+
+!---------------------------------
+!---------------------------------
+
+subroutine smooth_2d( x, n, y, method, weight, error )
+! 2 ǡˤĤ, ࡼ󥰤롼
+! , error ץϵǽƤʤ.
+  implicit none
+  real, intent(in) :: x(:,:)  ! ࡼ󥰤ǡ
+  integer, intent(in) :: n  ! ࡼαƶʻҿ(濴ޤ᤿)
+  real, intent(inout) :: y(size(x,1),size(x,2))  ! ࡼ󥰤줿ǡ
+  character(3), intent(in) :: method  ! ࡼ󥰤ˡ
+                            ! "SMP" = ñʿ, "OPT" = ץŤ
+                            ! "MAX" = , "MIN" = Ǿ
+  real, intent(in), optional :: weight(n,n)  ! method  "OPT" ξ
+                          ! weight(1,1) ü, weight(n,n) üȤƽŤ
+  real, intent(in), optional :: error  ! ̤
+  integer :: ix, iy, j, k, mx, my, half, val
+  real :: div_fact
+  real :: wg(n,n)
+
+  y=0.0
+  mx=size(x,1)
+  my=size(x,2)
+  half=(n-1)/2
+
+  select case (method(1:3))
+  case ("SMP")
+     do k=1,n
+        do j=1,n
+           wg(j,k)=1.0
+        end do
+     end do
+
+  case ("OPT")
+     if(present(weight))then
+        do k=1,n
+           do j=1,n
+              wg(j,k)=weight(j,k)
+           end do
+        end do
+     end if
+  end select
+
+  if(method(1:3)/="MIN".and.method(1:3)/="MAX")then
+!-- determining dividing factor
+
+     div_fact=0.0
+
+     do k=1,n
+        do j=1,n
+           if(wg(j,k)<0.0)then
+              div_fact=1.0
+              exit
+           else
+              div_fact=div_fact+wg(j,k)
+           end if
+        end do
+     end do
+
+!-- avoiding zero dividing
+
+     if(div_fact==0.0)then
+        div_fact=1.0
+     end if
+
+     do iy=half+1,my-half
+        do ix=half+1,mx-half
+           do k=1,n
+              do j=1,n
+                 y(ix,iy)=y(ix,iy)+x(ix-half-1+j,iy-half-1+k)*wg(j,k)
+              end do
+           end do
+
+           y(ix,iy)=y(ix,iy)/div_fact
+        end do
+     end do
+
+  else
+
+     select case (method(1:3))
+     case ("MAX")
+        do iy=half+1,my-half
+           do ix=half+1,mx-half
+              val=x(ix-half,iy-half)
+              do k=2,n
+                 do j=2,n
+                    if(val<x(ix-half-1+j,iy-half-1+k))then
+                       val=x(ix-half-1+j,iy-half-1+k)
+                    end if
+                 end do
+              end do
+              y(ix-half,iy-half)=val
+           end do
+        end do
+
+     case ("MIN")
+        do iy=half+1,my-half
+           do ix=half+1,mx-half
+              val=x(ix-half,iy-half)
+              do k=2,n
+                 do j=2,n
+                    if(val>x(ix-half-1+j,iy-half-1+k))then
+                       val=x(ix-half-1+j,iy-half-1+k)
+                    end if
+                 end do
+              end do
+              y(ix-half,iy-half)=val
+           end do
+        end do
+
+     end select
+
+  end if
+
+end subroutine smooth_2d
+
+!---------------------------------
+!---------------------------------
+
+subroutine smooth_3d( x, n, y, method, weight, error )
+! 3 ǡˤĤ, ࡼ󥰤롼
+! , error ץϵǽƤʤ.
+  implicit none
+  real, intent(in) :: x(:,:,:)  ! ࡼ󥰤ǡ
+  integer, intent(in) :: n  ! ࡼαƶʻҿ(濴ޤ᤿)
+  real, intent(inout) :: y(size(x,1),size(x,2),size(x,3))
+                            ! ࡼ󥰤줿ǡ
+  character(3), intent(in) :: method  ! ࡼ󥰤ˡ
+                            ! "SMP" = ñʿ, "OPT" = ץŤ
+                            ! "MAX" = , "MIN" = Ǿ
+  real, intent(in), optional :: weight(n,n,n)  ! method  "OPT" ξ
+  real, intent(in), optional :: error  ! ̤
+  integer :: ix, iy, iz, j, k, l, mx, my, mz, half, val
+  real :: div_fact
+  real :: wg(n,n,n)
+
+  y=0.0
+  mx=size(x,1)
+  my=size(x,2)
+  mz=size(x,3)
+  half=(n-1)/2
+
+  select case (method(1:3))
+  case ("SMP")
+     do l=1,n
+        do k=1,n
+           do j=1,n
+              wg(j,k,l)=1.0
+           end do
+        end do
+     end do
+
+  case ("OPT")
+     if(present(weight))then
+        do l=1,n
+           do k=1,n
+              do j=1,n
+                 wg(j,k,l)=weight(j,k,l)
+              end do
+           end do
+        end do
+     end if
+  end select
+
+  if(method(1:3)/="MIN".and.method(1:3)/="MAX")then
+!-- determining dividing factor
+
+     div_fact=0.0
+
+     do l=1,n
+        do k=1,n
+           do j=1,n
+              if(wg(j,k,l)<0.0)then
+                 div_fact=1.0
+                 exit
+              else
+                 div_fact=div_fact+wg(j,k,l)
+              end if
+           end do
+        end do
+     end do
+
+!-- avoiding zero dividing
+
+     if(div_fact==0.0)then
+        div_fact=1.0
+     end if
+
+     do iz=half+1,mz-half
+        do iy=half+1,my-half
+           do ix=half+1,mx-half
+              do l=1,n
+                 do k=1,n
+                    do j=1,n
+                       y(ix,iy,iz)=y(ix,iy,iz)  &
+  &                                +x(ix-half-1+j,iy-half-1+k,iz-half-1+l)  &
+  &                                *wg(j,k,l)
+                    end do
+                 end do
+              end do
+
+              y(ix,iy,iz)=y(ix,iy,iz)/div_fact
+           end do
+        end do
+     end do
+
+  else
+
+     select case (method(1:3))
+     case ("MAX")
+        do iz=half+1,mz-half
+           do iy=half+1,my-half
+              do ix=half+1,mx-half
+                 val=x(ix-half,iy-half,iz-half)
+                 do l=2,n
+                    do k=2,n
+                       do j=2,n
+                          if(val<x(ix-half-1+j,iy-half-1+k,iz-half-1+l))then
+                             val=x(ix-half-1+j,iy-half-1+k,iz-half-1+l)
+                          end if
+                       end do
+                    end do
+                 end do
+                 y(ix-half,iy-half,iz-half)=val
+              end do
+           end do
+        end do
+
+     case ("MIN")
+        do iz=half+1,mz-half
+           do iy=half+1,my-half
+              do ix=half+1,mx-half
+                 val=x(ix-half,iy-half,iz-half)
+                 do l=2,n
+                    do k=2,n
+                       do j=2,n
+                          if(val>x(ix-half-1+j,iy-half-1+k,iz-half-1+l))then
+                             val=x(ix-half-1+j,iy-half-1+k,iz-half-1+l)
+                          end if
+                       end do
+                    end do
+                 end do
+                 y(ix-half,iy-half,iz-half)=val
+              end do
+           end do
+        end do
+
+     end select
+
+  end if
+
+end subroutine smooth_3d
+
+!---------------------------------
+!---------------------------------
+
+subroutine Move_ave( x, n, y, error, offset )
 ! ưʿѤ׻롼
   implicit none
   real, intent(in) :: x(:)  ! ǡ
@@ -1473,33 +1920,56 @@ subroutine Move_ave( x, n, y, error )
   real, intent(inout) :: y(size(x))  ! ʿѲΥǡ.
                       ! ºݤ, y(1:n-1) ޤǤˤϥ.
   real, intent(in), optional :: error  ! »
-  integer :: nx, i
+  integer, intent(in), optional :: offset  ! ưʿѤ򳫻Ϥֹ.
+                      ! default = n
+  integer :: nx, i, ioff
   real :: tmp
 
   nx=size(x)
+  y=0.0
 
   if(nx<n.or.n<2)then
      write(*,*) "### ERROR ### (Move_ave)"
      write(*,*) "x(nx) : nx must be more than n or n must be more than 2."
+     write(*,*) "nx is ", nx, ", n is ", n, "."
      write(*,*) "STOP"
      stop
   end if
 
+  if(present(offset))then
+     if(offset>0)then
+        ioff=offset
+     else
+        write(*,*) "### ERROR ### (Move_ave)"
+        write(*,*) "offset must be more than 1."
+        write(*,*) "STOP"
+        stop
+     end if
+  else
+     ioff=n
+  end if
+
   if(present(error))then
      call Mean_1d( x(1:n), tmp, error )
-     y(1:n-1)=0.0
-     y(n)=tmp
+     if(ioff>2)then
+        y(1:ioff-1)=0.0
+        y(nx-n+ioff+1:nx)=0.0
+     end if
+     y(ioff)=tmp
 
-     do i=n+1,nx
-        y(i)=y(i-1)+(x(i)-x(i-n))/real(n)
+     do i=ioff+1,nx-n+ioff
+        y(i)=y(i-1)+(x(i+n-ioff)-x(i-ioff))/real(n)
      end do
   else
      call Mean_1d( x(1:n), tmp )
-     y(1:n-1)=0.0
-     y(n)=tmp
+     if(ioff>2)then
+        y(1:ioff-1)=0.0
+        y(nx-n+ioff+1:nx)=0.0
+     end if
+     y(ioff)=tmp
 
-     do i=n+1,nx
-        y(i)=y(i-1)+(x(i)-x(i-n))/real(n)
+     do i=ioff+1,nx-n+ioff
+        y(i)=y(i-1)+(x(i+n-ioff)-x(i-ioff))/real(n)
      end do
   end if
 
@@ -1508,6 +1978,65 @@ end subroutine Move_ave
 !---------------------------------
 !---------------------------------
 
+subroutine Move_anom( x, n, y, error, offset )
+! ưʿѤΥΥޥ׻롼
+  implicit none
+  real, intent(in) :: x(:)  ! ǡ
+  integer, intent(in) :: n  ! ʿѤȤ
+  real, intent(inout) :: y(size(x))  ! ʿѲΥǡ.
+                      ! ºݤ, y(1:n-1) ޤǤˤϥ.
+  real, intent(in), optional :: error  ! »
+  integer, intent(in), optional :: offset  ! ưʿѤ򳫻Ϥֹ.
+                      ! default = n
+  integer :: nx, i, ioff
+  real :: tmp, undef
+  real :: bar(size(x))
+
+  nx=size(x)
+  y=0.0
+
+  if(nx<n.or.n<2)then
+     write(*,*) "### ERROR ### (Move_anom)"
+     write(*,*) "x(nx) : nx must be more than n or n must be more than 2."
+     write(*,*) "nx is ", nx, ", n is ", n, "."
+     write(*,*) "STOP"
+     stop
+  end if
+
+  if(present(offset))then
+     if(offset>0)then
+        ioff=offset
+     else
+        write(*,*) "### ERROR ### (Move_anom)"
+        write(*,*) "offset must be more than 1."
+        write(*,*) "STOP"
+        stop
+     end if
+  else
+     ioff=n
+  end if
+
+  if(present(error))then
+     undef=error
+  else
+     undef=0.0
+  end if
+
+  call Move_ave( x, n, bar, error=undef, offset=ioff )
+
+  if(ioff>2)then
+     y(1:ioff-1)=0.0
+     y(nx-n+ioff+1:nx)=0.0
+  end if
+  do i=ioff,nx-n+ioff
+     y(i)=x(i)-bar(i)
+  end do
+
+end subroutine Move_anom
+
+!---------------------------------
+!---------------------------------
+
 subroutine Bubble_Sort_i( a, b, sig )
 ! Х֥륽ȤѤƿͥǡ sig ˥Ȥ.
   implicit none
